`define SLAVE_ADD_WRITE 8'ha2
`define SLAVE_ADD_READ 8'ha3

module pcf8563(CLOCK_50,Q_KEY,PCF8563_SCL,PCF8563_SDA,SEG7_SEG,SEG7_SEL);
input CLOCK_50;
input Q_KEY;
inout PCF8563_SCL,PCF8563_SDA;
output[7:0] SEG7_SEG;
output[2:0]	SEG7_SEL;

reg clk;
reg [7:0] SEG7_SEG;
reg[2:0]	SEG7_SEL;
reg PCF8563_SCL_hi_z;
reg PCF8563_SDA_hi_z;
reg PCF8563_SCL_reg;
reg PCF8563_SDA_reg;
reg[3:0] p_state;
reg[7:0] write_reg;
reg[7:0] read_reg;
reg[3:0] SEG7_SEG_reg;
reg[10:0] i;
reg[7:0] s_reg;
reg[7:0] m_reg;
reg[7:0] h_reg;
reg[7:0] d_reg;


reg[5:0] cnt;
reg[2:0] cnt1;
integer cnt2;
reg[15:0] cnt3;
reg[2:0] cnt4;

parameter prepare=0;
parameter idle=1;
parameter start=2;
parameter stop=3;
parameter write_data=4;
parameter wait_ack=5;
parameter error=6;
parameter read_data=7;
parameter ack=8;
parameter nack=9;


assign PCF8563_SCL=PCF8563_SCL_hi_z? 1'bz:1'b0;
assign PCF8563_SDA=PCF8563_SDA_hi_z? 1'bz:1'b0;


//div
always @(posedge CLOCK_50)
begin
if (Q_KEY)
begin
cnt<=cnt+1;
if (cnt==6'b100000)
begin     
cnt<=0;
clk<=~clk;
end
end
else
cnt<=0;
end


always @(posedge CLOCK_50)
PCF8563_SDA_reg<=PCF8563_SDA;


always @(posedge clk)
begin
if (!Q_KEY)
begin
p_state<=prepare;
cnt2<=0;
cnt3<=0;
cnt4<=0;
write_reg<=0;
end

else
begin

case(p_state)
prepare:
begin
PCF8563_SCL_hi_z<=1;
PCF8563_SDA_hi_z<=1;
cnt2<=cnt2+1;
if(cnt2==10)
begin
cnt2<=0;
p_state<=idle;
end
else
p_state<=prepare;
end



idle:
begin
cnt3<=cnt3+1;
case (cnt3)

16'd0:p_state<=start;
16'd1:begin p_state<=write_data;write_reg<=`SLAVE_ADD_WRITE ; end
16'd2:begin p_state<=write_data;write_reg<=8'h02 ; end
16'd3:begin p_state<=write_data;write_reg<=8'h00 ; end  //s set
16'd4:begin p_state<=write_data;write_reg<=8'h4 ; end  //m set
16'd5:begin p_state<=write_data;write_reg<=8'h19 ; end  //h set
16'd6:begin p_state<=write_data;write_reg<=8'h30 ; end  //d set
16'd7:begin p_state<=write_data;write_reg<=8'h06 ; end  //week
16'd8:begin p_state<=write_data;write_reg<=8'h01 ; end  //month
16'd9:begin p_state<=write_data;write_reg<=8'h10 ; end  //year
16'd10:p_state<=start;
16'd11:begin p_state<=write_data;write_reg<=`SLAVE_ADD_WRITE ; end
16'd12:begin p_state<=write_data;write_reg<=8'h02 ; end
16'd13:p_state<=start;
16'd14:begin p_state<=write_data;write_reg<=`SLAVE_ADD_READ ; end
16'd15:p_state<=read_data;
16'd16:p_state<=ack;
16'd17:begin p_state<=read_data; s_reg<=read_reg; end
16'd18:p_state<=ack;
16'd19:begin p_state<=read_data; m_reg<=read_reg; end
16'd20:p_state<=ack;
16'd21:begin p_state<=read_data; h_reg<=read_reg; end
16'd22:p_state<=nack;
16'd23:begin p_state<=stop;   d_reg<=read_reg; end
16'd24:begin   cnt3<=10; end



endcase

end


start:
begin
cnt2<=cnt2+1;
case (cnt2)
0:begin PCF8563_SCL_hi_z<=1;
        PCF8563_SDA_hi_z<=1; 
        end
1:begin PCF8563_SCL_hi_z<=1;
        PCF8563_SDA_hi_z<=0; 
        end
2:begin PCF8563_SCL_hi_z<=0;
        PCF8563_SDA_hi_z<=0; 
        end

4:begin cnt2<=0;
        p_state<=idle;
        end
//default:cnt2<=0;
endcase

end




stop:
begin
cnt2<=cnt2+1;
case (cnt2)
1: PCF8563_SDA_hi_z<=0; 
2: PCF8563_SCL_hi_z<=1; 
3: PCF8563_SDA_hi_z<=1; 
4: begin
   cnt2<=0;
    p_state<=idle;
    end
 endcase
 end


write_data:
begin
cnt2<=cnt2+1;
case (cnt2)

1: PCF8563_SDA_hi_z<=write_reg[7-cnt4];
2: PCF8563_SCL_hi_z<=1; 
4: PCF8563_SCL_hi_z<=0; 
5: begin
   cnt2<=0;
   cnt4<=cnt4+1;
   if (cnt4==3'd7)
   p_state<=wait_ack;
   //cnt4<=0;
   else 
   p_state<=write_data;
   end
endcase

end


wait_ack:
begin
cnt2<=cnt2+1;
case (cnt2)

0:PCF8563_SDA_hi_z<=1; 
1:PCF8563_SCL_hi_z<=1; 
4:begin
  if (!PCF8563_SDA_reg)
  begin
  PCF8563_SCL_hi_z<=0; 

  p_state<=idle;
  cnt2<=0;
  end
  else
  begin
  p_state<=error;
  cnt2<=0;
  
  end
  end
endcase
  end

 
  
 error:
 begin 
  p_state<=error;
  end

read_data:
begin
cnt2<=cnt2+1;
case (cnt2)

0: PCF8563_SDA_hi_z<=1;
1: PCF8563_SCL_hi_z<=1;
2: read_reg[7-cnt4]<=PCF8563_SDA_reg;
3: PCF8563_SCL_hi_z<=0; 
4: begin
   cnt2<=0;
   cnt4<=cnt4+1;
   if (cnt4==3'd7)
   p_state<=idle;
   else 
   p_state<=read_data;
   end
endcase
end


ack:
begin

cnt2<=cnt2+1;
case (cnt2)

0:PCF8563_SDA_hi_z<=0; 
1:PCF8563_SCL_hi_z<=1; 

4:begin
  cnt2<=0;
  p_state<=idle;
  PCF8563_SCL_hi_z<=0; 
  end
  
endcase
  end
  
  nack:
begin

cnt2<=cnt2+1;
case (cnt2)

0:PCF8563_SDA_hi_z<=1; 
1:PCF8563_SCL_hi_z<=1; 

4:begin
  cnt2<=0;
  p_state<=idle;
  PCF8563_SCL_hi_z<=0; 
  end
  
endcase
  end



default p_state<='bx;
endcase
end
end

always @(posedge clk )
begin
if (Q_KEY)
begin
i<=i+1;
if (i==11'b111_1111_1111) 
begin 
 i<=0;

 cnt1<=cnt1+1;
  
 if (cnt1==3'b111)
   begin 
  cnt1<=3'b000;
 end
 end
 end
 end

always @(cnt1)
 begin
case(cnt1)				  
		
		3'd0:SEG7_SEL = 3'b111;			//ѡһʾ
		3'd1:SEG7_SEL = 3'b110;			//ѡڶʾ
		3'd2:SEG7_SEL = 3'b101;			//ѡʾ
		3'd3:SEG7_SEL = 3'b100;			//ѡĸʾ
		3'd4:SEG7_SEL = 3'b011;			//ѡʾ
		3'd5:SEG7_SEL = 3'b010;			//ѡʾ
		3'd6:SEG7_SEL = 3'b001;			//ѡ߸ʾ
		3'd7:SEG7_SEL = 3'b000;			//ѡڰ˸ʾ
	endcase	
	case(cnt1)						//ѡɨʾ
		3'd0:SEG7_SEG_reg = s_reg[3:0];			
		3'd1:SEG7_SEG_reg = s_reg[7:4];		
		3'd2:SEG7_SEG_reg = m_reg[3:0];			
		3'd3:SEG7_SEG_reg = {1'b0,m_reg[6:4]};	 
		3'd4:SEG7_SEG_reg = h_reg[3:0];		
		3'd5:SEG7_SEG_reg = {2'b0,h_reg[5:4]};					
		3'd6:SEG7_SEG_reg = d_reg[3:0];		
		3'd7:SEG7_SEG_reg = {2'b0,d_reg[5:4]};		
    endcase
	
end 

always @(SEG7_SEG_reg)
begin	
	
	case(SEG7_SEG_reg)
		4'h0:SEG7_SEG = 8'hc0;				//ʾ0
		4'h1:SEG7_SEG = 8'hf9;				//ʾ1
		4'h2:SEG7_SEG = 8'ha4;				//ʾ2
		4'h3:SEG7_SEG = 8'hb0;				//ʾ3
		4'h4:SEG7_SEG = 8'h99;				//ʾ4
		4'h5:SEG7_SEG = 8'h92;				//ʾ5
		4'h6:SEG7_SEG = 8'h82;				//ʾ6
		4'h7:SEG7_SEG = 8'hf8;				//ʾ7
		4'h8:SEG7_SEG = 8'h80;				//ʾ8
		4'h9:SEG7_SEG = 8'h90;				//ʾ9
		4'ha:SEG7_SEG = 8'h88;				//ʾa
		4'hb:SEG7_SEG = 8'h83;				//ʾb
		4'hc:SEG7_SEG = 8'hc6;				//ʾc
		4'hd:SEG7_SEG = 8'ha1;				//ʾd
		4'he:SEG7_SEG = 8'h86;				//ʾe
		4'hf:SEG7_SEG = 8'h8e;	
	endcase
 
 end 

endmodule

